using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace licenser
{
    class Program
    {
        static Stack<string> dirs = new Stack<string>();
        static string[] license = new string[] 
        { 
            "// This File is part of the \"{0}\" Project", 
            "// ",
            "// Copyright  2009 sometmes@gmail.com",
            "// All rights reserved",
            "// ",
            "// This library is free software; you can redistribute it and/or modify",
            "// it under the terms of the GNU Lesser General Public License as published",
            "// by the Free Software Foundation; either version 2.1 of the License, or",
            "// (at your option) any later version.",
            "// This program is distributed in the hope that it will be useful,",
            "// but WITHOUT ANY WARRANTY; without even the implied warranty of",
            "// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the",
            "// GNU General Public License for more details.",
            "// ",
            "// You should have received a copy of the GNU General Public License",
            "// along with this program.  If not, see <http://www.gnu.org/licenses/>.",
            ""
        };

        const string companymatch = "[assembly: AssemblyCompany(";
        const string company = "[assembly: AssemblyCompany(\"sometmes@gmail.com\")]";

        const string copyrightmatch = "[assembly: AssemblyCopyright(";
        const string copyright = "[assembly: AssemblyCopyright(\"sometmes@gmail.com  2009\")]";

        static void Main(string[] args)
        {
            if (args.Length < 2)
                throw new ArgumentException("You must supply path and project name as parameters");
            string root = Path.GetFullPath(Path.Combine(Environment.CurrentDirectory, args[0]));
            if (!Directory.Exists(root))
                throw new ArgumentException("Path not exists: " + root);
            dirs.Push(root);
            string project = args[1];
            license[0] = string.Format(license[0], project);
            while (dirs.Count > 0)
            {
                string dir = dirs.Pop();
                if (dir.EndsWith("\\_svn") || dir.EndsWith("\\.svn") || dir.EndsWith("\\licenser") || dir.EndsWith("\\ICSharpCode.TextEditor"))
                    continue;
                string[] subdirs = Directory.GetDirectories(dir);
                foreach (string subdir in subdirs) dirs.Push(subdir);
                string[] files = Directory.GetFiles(dir);
                foreach (string file in files)
                {
                    if (Path.GetExtension(file).Equals(".cs", StringComparison.OrdinalIgnoreCase))
                    {
                        string[] filecontent = File.ReadAllLines(file);
                        int endlicense = 0;
                        while (endlicense < filecontent.Length && filecontent[endlicense].StartsWith("//"))
                            endlicense++;
                        if (endlicense > 0)
                        {
                            while (filecontent[endlicense] == "")
                                endlicense++;
                        }
                        for (int i = 0; i < filecontent.Length; i++)
                        {
                            string contentline = filecontent[i];
                            if (contentline.StartsWith(copyrightmatch))
                                filecontent[i] = copyright;
                            if (contentline.StartsWith(companymatch))
                                filecontent[i] = company;
                        }

                        string[] newcontent = new string[license.Length + filecontent.Length - endlicense];
                        license.CopyTo(newcontent, 0);
                        Array.Copy(filecontent, endlicense, newcontent, license.Length, filecontent.Length - endlicense);
                        File.WriteAllLines(file, newcontent);
                    }
                }
            }
            Console.WriteLine("Finished. Press <return> to end.");
            Console.ReadLine();
        }
    }
}
